# -*- coding:Utf-8 -*-
import es, psyco, random, gamethread, weaponlib
psyco.full()

# try to import simplemenulib
try: import simplemenulib
except ImportError: raise ImportError, "dm_python: module simplemenulib is required (http://addons.eventscripts.com/addons/view/simplemenu)"

# if Source Python Extensions is usable, import it...
try:
    from spe.games import cstrike as es_spe
    SourcePythonExtensions = True
except ImportError:
    es.dbgmsg(1, "dm_python: Source Python Extensions is not installed and it will not be used")
    es.dbgmsg(1, "dm_python: you can get this plugin at this following adress http://forums.eventscripts.com/viewtopic.php?t=29657")
    SourcePythonExtensions = False


# if Source Native Tools is usable, import it...
try:
    import nativetools as es_nt
    NativeTools = True
except ImportError:
    es.dbgmsg(1, "dm_python: Native Tools is not installed and it will not be used")
    es.dbgmsg(1, "dm_python: you can get this plugin at this following adress http://forums.eventscripts.com/viewtopic.php?t=32582")
    NativeTools = False

# import modules of dm_python script and associated objects
import dm_player, dm_config, dm_weapon, dm_spawns, dm_votemap, dm_entities, dm_language, dm_usermenu, dm_adminmenu
from dm_config import dmConfig
from dm_weapon import dmWeapon
from dm_spawns import dmSpawns
from dm_votemap import dmVotemap


esInfo = es.AddonInfo()
esInfo.author = "DoCky"
esInfo.version = "2.408p"
esInfo.date = "2010/12/23"
esInfo.basename = "dm_python"
esInfo.name = "Deathmatch Python"
esInfo.url = "http://addons.eventscripts.com/addons/user/11751"

version = es.ServerVar("dm_python", "", " Created by DoCky -")
version.set(esInfo.version)
version.makepublic()
roundstat = False
playerdrop = []


############################
# FUNCTION
""" check if dm_extension id loaded """
def extensionLoaded():
    # merci Nicolous...
    basename = "dm_python/dm_extension"["dm_python/dm_extension".rfind("/") + 1:]
    pyname = "%s.%s" % ("dm_python/dm_extension".replace("/","."), basename)
    for addon in es.addons.AddonList:
        if addon.__name__ == pyname: return True
    return False

""" initialization """
def init():
    global playerdrop
    playerdrop = []

    # update config and search spawnpoint
    dmConfig.update()
    dmSpawns.search(True)
    # initialize votemap and weapon system
    dmVotemap.newmap()
    dmWeapon.order()
    dmWeapon.read()

    # load language and create menu
    dm_language.load()
    dm_usermenu.create()
    dm_adminmenu.create()

    # register all player in the server
    for userid in es.getUseridList():
        player = dm_player.player(userid)
        if es.getplayerteam(userid) > 1: gamethread.queue(player.respawn)

    # remove objectives, weapons and items on the ground
    dm_entities.removeObjectivesFromTheMap()
    dm_entities.removeWeaponsAndItemsOnTheGround(False)


############################
# COMMANDS
""" remove weapons and items command """
def serverRemoveWeaponsAndItem():
    dm_entities.removeWeaponsAndItemsOnTheGround(bool(es.getargv(1)))

""" check if player has respawned """
def playerRespawned():
    userid = es.getargv(1)
    player = dm_player.player(userid)
    # if player is not alive
    if es.getplayerprop(userid, "CBasePlayer.pl.deadflag"):
        # if attempts to respawn the player are more than 5
        if player.respawntry > 4: es.dbgmsg(0, 'dm_python: respawning of userid "%s" failed after five retry' % userid)
        # else try to respawn player again
        else: gamethread.delayedname(1, player.threadRespawn, player.respawn)

""" intercept drop client command """
def playerDrop(userid, arguments):
    if arguments[0].lower() == "drop":
        if not userid in playerdrop:
            playerdrop.append(userid)
            gamethread.queue(dm_entities.removeWeaponsAndItemsOnTheGround, True)
            # "drop" command allowed once per second (per player)
            gamethread.delayedname(1, "_dm_python_playerdrop_" + str(userid), playerdrop.remove, userid)
            return True
        return False
    return True

""" menu command """
def deathmatchMenu():
    userid = es.getcmduserid()
    if not es.getplayerprop(userid, "CBasePlayer.pl.deadflag"):
        menu = simplemenulib.active(userid)["name"]
        if menu in simplemenulib.getlist("_dm_python_list_menu"): es.tell(userid, "#multi", dm_language.message(userid, "deathmatch menu opened"))
        else:
            if int(dmConfig.style) == 1: simplemenulib.send(userid, "_dm_python_categorie_menu")
            elif dmWeapon.secondary: simplemenulib.send(userid, "_dm_python_secondary_menu")
            else: simplemenulib.send(userid, "_dm_python_primary_menu")
        dm_player.player(userid).firstspawn = False

""" admin command """
def administrationMenu():
    userid = es.getcmduserid()
    steamid = es.getplayersteamid(userid)
    if steamid in dmConfig:
        menu = simplemenulib.active(userid)["name"]
        if menu in simplemenulib.getlist("_dm_python_list_menu"): es.tell(userid, "#multi", dm_language.message(userid, "deathmatch menu opened"))
        else: dm_adminmenu.administrationUpdate(userid)

""" change convars value """
def cvarValue():
    userid = es.getcmduserid()
    # if player is an admin
    if es.getplayersteamid(userid) in dmConfig:
        # get new value to submit
        value = es.getargv(1)
        # if value isn't emptie
        if value:
            # file will not rewritted (for now), get name of variable contening ServerVar instance
            write, name = False, es.getargv(0).replace("_dp_cvar_", "")
            # get ServerVar instance, type of value and supported value, name of the CFG
            cvar, values, cfgname = dmConfig[name]

            # if the value must be an int
            if isinstance(values, int):
                try:
                    # check type of value to submit
                    value = int(value)
                    # if value can be set (greather than X)
                    if value >= values:
                        # set value and save cfg
                        cvar.set(value)
                        dmConfig.saveCfg(cfgname)
                        # change timeout of deathmach menu if the config has change
                        if name == "timeout":
                            for menu in simplemenulib.getlist("_dm_python_deathmatch_menu"): simplemenulib.find(menu).timeout("view", value)
                    else: es.tell(userid, "#multi", dm_language.message(userid, "escmenu imputbox greaterint", {"value" : "#lightgreen" + str(values)}))
                except ValueError: es.tell(userid, "#multi", dm_language.message(userid, "escmenu imputbox wrongint"))

            # if the value must be an float
            elif isinstance(values, float):
                try:
                    # check type of value to submit
                    fvalue = float(value)
                    # if value can be set (greather than X)
                    if fvalue >= values:
                        # set value and save cfg
                        cvar.set(value)
                        dmConfig.saveCfg(cfgname)
                    else: es.tell(userid, "#multi", dm_language.message(userid, "escmenu imputbox greaterint", {"value" : "#lightgreen" + str(values)}))
                except ValueError: es.tell(userid, "#multi", dm_language.message(userid, "escmenu imputbox wrongint"))

            # if the value must be in a specific range
            elif isinstance(values, tuple):
                # check if values can be set
                if value in values:
                    # set value and save cfg
                    cvar.set(value)
                    dmConfig.saveCfg(cfgname)

                    # name is "style": recreate style of menu
                    if name == "style": dm_usermenu.create()
                    # name is "extension": un/load dm_extension and update admin menu
                    elif name == "extension":
                        menu = simplemenulib.find("_dm_python_administration_menu")
                        if value == "1":
                            menu.modchoice("admin extension option", "admin extension option", underline = True)
                            if not extensionLoaded(): es.load("dm_python/dm_extension")
                        else:
                            menu.modchoice("admin extension option", "admin extension option", underline = False)
                            if extensionLoaded(): es.unload("dm_python/dm_extension")
                        menu.update("#all")

                else: es.tell(userid, "#multi", dm_language.message(userid, "escmenu imputbox wrongvalue", {"values" : "#default|".join(map(lambda value: "#lightgreen" + value, values))}))

            # if the value could be anything
            else:
                # set value and save cfg
                write = True
                cvar.set(value)
                dmConfig.saveCfg(cfgname)
                # name is "admin|command|randguns": clear old commands and create all news
                if name == "admin" or name == "command" or name == "randguns": dmConfig.createCmd(name)

""" add a map in maplist """
def addMapInList():
    userid = es.getcmduserid()
    # if player is admin
    if es.getplayersteamid(userid) in dmConfig:
        value = es.getargv(1)
        # check if the arguments is not emptie
        if value:
            # if the map does not exists
            if not es.exists("map", value): es.tell(userid, "#multi", dm_language.message(userid, "deathmatch map invalid", {"mapname" : value}))
            else:
                es.tell(userid, "#multi", dm_language.message(userid, "deathmatch map added", {"mapname" : value}))
                # add the map in maplist.txt
                dm_votemap.maplist.addlist(value, False)
                # update the dmVotemap list
                dmVotemap.read()

""" un/activate random weapons """
def randomWeapons():
    userid = es.getcmduserid()
    player = dm_player.player(userid)
    # invert "random weapon" status of player
    if player.randguns:
        player.randguns = False
        es.tell(userid, "#multi", dm_language.message(userid, "deathmatch randguns unactivated"))
    else:
        player.randguns = True
        es.tell(userid, "#multi", dm_language.message(userid, "deathmatch randguns activated"))


############################
# EVENT
def load():
    # initialize the system
    init()
    es.addons.registerClientCommandFilter(playerDrop)
    # create internals command for check if player has been spawned correctly
    if not es.exists("command", "_dp_player_respawn"):
        es.regcmd("_dp_player_respawn", "dm_python/playerRespawned", "[dm_python] internal command for check has respawned")
    # create public command for remove weapons and items on the ground
    if not es.exists("command", "deathmatch_python_removeIDLE"):
        es.regcmd("deathmatch_python_removeIDLE", "dm_python/serverRemoveWeaponsAndItem", "[dm_python] server command for remove weapons and items at ground")

    es.regclientcmd("_dp_cvar_admin", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_command", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_randguns", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_style", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_timeout", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_plugin", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_protection", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_random", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_respawn", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_dissolver", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_extension", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")

    es.regclientcmd("_dp_cvar_assaultsuit", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_nightvision", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_hegrenade", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_flashbang", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_smokegrenade", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")

    es.regclientcmd("_dp_cvar_kills", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_time", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_maxmap", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_lastmaps", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_add_mapname", "dm_python/addMapInList", "[dm_python] internal command for in-game configuration")

    es.regclientcmd("_dp_cvar_reloadclip", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")
    es.regclientcmd("_dp_cvar_giveammo", "dm_python/cvarValue", "[dm_python] internal command for in-game configuration")

def unload():
    # close all systems, delete all menus and commands, unload extension
    dmSpawns.save()
    dm_player.unload()
    dmConfig.deleteCmd()
    simplemenulib.dellist("_dm_python_list_menu")
    if extensionLoaded(): es.unload("dm_python/dm_extension")

    es.addons.unregisterClientCommandFilter(playerDrop)
    gamethread.cancelDelayed("_dm_python_remove_weapons_and_items")
    for userid in playerdrop: gamethread.cancelDelayed("_dm_python_playerdrop_" + str(userid))

    es.unregclientcmd("_dp_cvar_admin")
    es.unregclientcmd("_dp_cvar_command")
    es.unregclientcmd("_dp_cvar_randguns")
    es.unregclientcmd("_dp_cvar_style")
    es.unregclientcmd("_dp_cvar_timeout")
    es.unregclientcmd("_dp_cvar_plugin")
    es.unregclientcmd("_dp_cvar_protection")
    es.unregclientcmd("_dp_cvar_random")
    es.unregclientcmd("_dp_cvar_respawn")
    es.unregclientcmd("_dp_cvar_dissolver")
    es.unregclientcmd("_dp_cvar_extension")

    es.unregclientcmd("_dp_cvar_assaultsuit")
    es.unregclientcmd("_dp_cvar_nightvision")
    es.unregclientcmd("_dp_cvar_hegrenade")
    es.unregclientcmd("_dp_cvar_flashbang")
    es.unregclientcmd("_dp_cvar_smokegrenade")

    es.unregclientcmd("_dp_cvar_kills")
    es.unregclientcmd("_dp_cvar_time")
    es.unregclientcmd("_dp_cvar_maxmap")
    es.unregclientcmd("_dp_cvar_lastmaps")
    es.unregclientcmd("_dp_add_mapname")

    es.unregclientcmd("_dp_cvar_reloadclip")
    es.unregclientcmd("_dp_cvar_giveammo")

def es_map_start(event):
    # save config and spawnpoints, initialize the system
    dmConfig.saveCfg()
    dmSpawns.save()
    init()

def round_end(event):
    # set round status
    global roundstat
    roundstat = False
    # the game (re)start: initiate vote
    if event["reason"] == "16": dmVotemap.islaunched = False

def round_start(event):
    # set round status
    global roundstat
    roundstat = True
    # remove objectives, weapons and items on the ground
    dm_entities.removeObjectivesFromTheMap()
    dm_entities.removeWeaponsAndItemsOnTheGround(False)

def player_disconnect(event):
    userid = event["userid"]
    # remove player
    try: del dm_player.dmPlayer[int(userid)]
    except KeyError: es.dbgmsg(2, 'dm_python: userid "%s" was not registered' % event["userid"])
    try:
        playerdrop.remove(userid)
        gamethread.cancelDelayed("_dm_python_playerdrop_" + userid)
    except ValueError: es.dbgmsg(2, 'dm_python: userid "%s" was not in playerdrop list' % event["userid"])

def player_team(event):
    # spawn player if he has to be
    if int(event["team"]) > 1: gamethread.delayed(2, dm_player.player(event["userid"]).respawn)

def player_spawn(event):
    userid = int(event["userid"])
    if not es.getplayerprop(userid, "CBasePlayer.pl.deadflag"):
        player = dm_player.player(userid)
        # player spawn. give weapons, items, etc...
        gamethread.queue(player.spawn, event["es_userteam"])

        # check, send or update menu
        if simplemenulib.isqueued(userid, "_dm_python_administration_menu", 1): dm_adminmenu.administrationUpdate(userid)
        elif simplemenulib.isqueued(userid, "_dm_python_spawnpoint_menu", 1): dm_adminmenu.spawnpointUpdate(userid, update = True, page = 1)
        elif simplemenulib.isqueued(userid, "_dm_python_spawnpoint_menu", 2): dm_adminmenu.spawnpointUpdate(userid, update = True, page = 2)
        elif player.firstspawn:
            menu = simplemenulib.active(userid)["name"]
            if not menu in simplemenulib.getlist("_dm_python_list_menu"):
                if int(dmConfig.style) == 1: simplemenulib.send(userid, "_dm_python_categorie_menu")
                elif dmWeapon.secondary: simplemenulib.send(userid, "_dm_python_secondary_menu")
                elif dmWeapon.primary: simplemenulib.send(userid, "_dm_python_primary_menu")
            player.firstspawn = False

def player_death(event):
    userid = event["userid"]
    # player death. remove weapons, items, etc...
    player = dm_player.player(userid).death()
    # check kills for the vote
    try: dmVotemap.check(int(event["es_attackerkills"]) + 1)
    except ValueError: pass

    # update menu (for admin)
    if simplemenulib.isqueued(userid, "_dm_python_administration_menu", 1): dm_adminmenu.administrationUpdate(userid)
    elif simplemenulib.isqueued(userid, "_dm_python_spawnpoint_menu", 1):
        es.tell(userid, "#multi", dm_language.message(userid, "admin alive message"))
        dm_adminmenu.spawnpointUpdate(userid, update = True)

def item_pickup(event):
    userid, item = event["userid"], event["item"]
    if item == "c4": dm_entities.removeEntity("weapon_c4", userid)
    else:
        # if item is not allowed
        if not item in dmWeapon:
            itemInfo = weaponlib.getWeapon(item)
            # if the item is a weapon
            if itemInfo:
                # remove weapons from the map
                dm_entities.removeEntity(itemInfo.name, userid)
                es.tell(userid, "#multi", dm_language.message(userid, "deathmatch restricted weapon", {"weapon" : item}))

def hegrenade_detonate(event):
    # check if user can get a new grenade
    dm_player.player(event["userid"]).grenade("he")

def flashbang_detonate(event):
    # check if user can get a new grenade
    dm_player.player(event["userid"]).grenade("fb")

def smokegrenade_detonate(event):
    # check if user can get a new grenade
    dm_player.player(event["userid"]).grenade("sg")
